@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -63,7 +63,7 @@
 #include "matcher-ac.h"
 #include "readdb.h"
 
-// shared
+// common
 #include "output.h"
 #include "optparser.h"
 #include "misc.h"
@@ -75,12 +75,15 @@
 #include "shared.h"
 #include "scanner.h"
 
+#ifdef _WIN32
+#include "service.h"
+#endif
+
 #include <sys/types.h>
 #ifndef WIN32
 #include <sys/wait.h>
 #endif
 
-
 short debug_mode = 0, logok = 0;
 short foreground = -1;
 
@@ -89,14 +92,19 @@ static void help(void)
     printf("\n");
     printf("                      Clam AntiVirus: Daemon %s\n", get_version());
     printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-    printf("           (C) 2021 Cisco Systems, Inc.\n");
+    printf("           (C) 2022 Cisco Systems, Inc.\n");
     printf("\n");
     printf("    clamd [options]\n");
     printf("\n");
     printf("    --help                   -h             Show this help\n");
     printf("    --version                -V             Show version number\n");
+#ifdef _WIN32
+    printf("    --install-service                       Install Windows Service\n");
+    printf("    --uninstall-service                     Uninstall Windows Service\n");
+#endif
     printf("    --foreground             -F             Run in foreground; do not daemonize\n");
     printf("    --debug                                 Enable debug mode\n");
+    printf("    --log=FILE               -l FILE        Log into FILE\n");
     printf("    --config-file=FILE       -c FILE        Read configuration from FILE\n");
     printf("\n");
     printf("Pass in - as the filename for stdin.\n");
@@ -126,8 +134,10 @@ int main(int argc, char **argv)
 #ifndef _WIN32
     struct passwd *user = NULL;
     struct sigaction sa;
-    struct rlimit rlim;
     int dropPrivRet = 0;
+#endif
+#if defined(C_LINUX) || (defined(RLIMIT_DATA) && defined(C_BSD))
+    struct rlimit rlim;
 #endif
     time_t currtime;
     const char *dbdir, *cfgfile;
@@ -144,9 +154,9 @@ int main(int argc, char **argv)
 #ifdef C_LINUX
     STATBUF sb;
 #endif
-    pid_t mainpid = 0;
-    mode_t old_umask = 0;
-    const char * user_name = NULL;
+    pid_t mainpid         = 0;
+    mode_t old_umask      = 0;
+    const char *user_name = NULL;
 
     if (check_flevel())
         exit(1);
@@ -156,8 +166,8 @@ int main(int argc, char **argv)
     sa.sa_handler = SIG_IGN;
     sigaction(SIGHUP, &sa, NULL);
     sigaction(SIGUSR2, &sa, NULL);
-    if(!setlocale(LC_CTYPE, "")) {
-       mprintf("^Failed to set locale\n");
+    if (!setlocale(LC_CTYPE, "")) {
+        mprintf("^Failed to set locale\n");
     }
 #endif
 
@@ -214,7 +224,7 @@ int main(int argc, char **argv)
     }
     free(pt);
 
-    if ((opt = optget(opts, "User"))->enabled){
+    if ((opt = optget(opts, "User"))->enabled) {
         user_name = opt->strarg;
     }
 
@@ -252,48 +262,47 @@ int main(int argc, char **argv)
         logg_file = NULL;
     }
 
-
 #ifndef WIN32
-        /* fork into background */
-        if (foreground == -1) {
-            if (optget(opts, "Foreground")->enabled) {
-                foreground = 1;
-            } else {
-                foreground = 0;
+    /* fork into background */
+    if (foreground == -1) {
+        if (optget(opts, "Foreground")->enabled) {
+            foreground = 1;
+        } else {
+            foreground = 0;
+        }
+    }
+    if (foreground == 0) {
+        int daemonizeRet = 0;
+#ifdef C_BSD
+        /* workaround for OpenBSD bug, see https://wwws.clamav.net/bugzilla/show_bug.cgi?id=885 */
+        for (ret = 0; (unsigned int)ret < nlsockets; ret++) {
+            if (fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) | O_NONBLOCK) == -1) {
+                logg("!fcntl for lsockets[] failed\n");
+                close(lsockets[ret]);
+                ret = 1;
+                break;
             }
         }
-        if (foreground == 0) {
-            int daemonizeRet = 0;
-#ifdef C_BSD
-            /* workaround for OpenBSD bug, see https://wwws.clamav.net/bugzilla/show_bug.cgi?id=885 */
-            for (ret = 0; (unsigned int)ret < nlsockets; ret++) {
-                if (fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) | O_NONBLOCK) == -1) {
-                    logg("!fcntl for lsockets[] failed\n");
-                    close(lsockets[ret]);
-                    ret = 1;
-                    break;
-                }
-            }
-#endif
-            gengine = engine;
-            atexit(free_engine);
-            daemonizeRet = daemonize_parent_wait(user_name, logg_file);
-            if (daemonizeRet < 0){
-                logg("!daemonize() failed: %s\n", strerror(errno));
-                return 1;
-            }
-            gengine = NULL;
-#ifdef C_BSD
-            for (ret = 0; (unsigned int)ret < nlsockets; ret++) {
-                if (fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) & ~O_NONBLOCK) == -1) {
-                    logg("!fcntl for lsockets[] failed\n");
-                    close(lsockets[ret]);
-                    ret = 1;
-                    break;
-                }
-            }
 #endif
+        gengine = engine;
+        atexit(free_engine);
+        daemonizeRet = daemonize_parent_wait(user_name, logg_file);
+        if (daemonizeRet < 0) {
+            logg("!daemonize() failed: %s\n", strerror(errno));
+            return 1;
         }
+        gengine = NULL;
+#ifdef C_BSD
+        for (ret = 0; (unsigned int)ret < nlsockets; ret++) {
+            if (fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) & ~O_NONBLOCK) == -1) {
+                logg("!fcntl for lsockets[] failed\n");
+                close(lsockets[ret]);
+                ret = 1;
+                break;
+            }
+        }
+#endif
+    }
 
 #endif
 
@@ -301,7 +310,7 @@ int main(int argc, char **argv)
     mainpid = getpid();
     if ((opt = optget(opts, "PidFile"))->enabled) {
         FILE *fd;
-        old_umask = umask(0002);
+        old_umask = umask(0022);
         if ((fd = fopen(opt->strarg, "w")) == NULL) {
             //logg("!Can't save PID in file %s\n", opt->strarg);
             logg("!Can't save PID to file %s: %s\n", opt->strarg, strerror(errno));
@@ -321,10 +330,10 @@ int main(int argc, char **argv)
         /*If the file has already been created by a different user, it will just be
          * rewritten by us, but not change the ownership, so do that explicitly.
          */
-        if (0 == geteuid()){
-            struct passwd * pw = getpwuid(0);
-            int ret = lchown(opt->strarg, pw->pw_uid, pw->pw_gid);
-            if (ret){
+        if (0 == geteuid()) {
+            struct passwd *pw = getpwuid(0);
+            int ret           = lchown(opt->strarg, pw->pw_uid, pw->pw_gid);
+            if (ret) {
                 logg("!Can't change ownership of PID file %s '%s'\n", opt->strarg, strerror(errno));
                 exit(2);
             }
@@ -332,6 +341,22 @@ int main(int argc, char **argv)
 #endif /* _WIN32 */
     }
 
+#ifdef _WIN32
+
+    if (optget(opts, "install-service")->enabled) {
+        svc_install("clamd", "ClamAV ClamD",
+                    "Provides virus scanning facilities for ClamAV");
+        optfree(opts);
+        return 0;
+    }
+
+    if (optget(opts, "uninstall-service")->enabled) {
+        svc_uninstall("clamd", 1);
+        optfree(opts);
+        return 0;
+    }
+#endif
+
     /* drop privileges */
 #ifndef _WIN32
     dropPrivRet = drop_privileges(user_name, logg_file);
@@ -620,6 +645,13 @@ int main(int argc, char **argv)
             logg("#Max A-C depth set to %u\n", (unsigned int)opt->numarg);
         }
 
+#ifdef _WIN32
+        if (optget(opts, "daemon")->enabled) {
+            cl_engine_set_clcb_sigload(engine, svc_checkpoint, NULL);
+            svc_register("clamd");
+        }
+#endif
+
         if ((ret = cl_load(dbdir, engine, &sigs, dboptions))) {
             logg("!%s\n", cl_strerror(ret));
             ret = 1;
@@ -741,7 +773,7 @@ int main(int argc, char **argv)
             }
 
             if (chmod(optget(opts, "LocalSocket")->strarg, sock_mode & 0666)) {
-                logg("!Cannot set socket permission to %s\n", optget(opts, "LocalSocketMode")->strarg);
+                logg("!Cannot set socket permission for %s to %3o\n", optget(opts, "LocalSocket")->strarg, sock_mode & 0666);
                 ret = 1;
                 break;
             }
@@ -782,12 +814,17 @@ int main(int argc, char **argv)
              * now, since everything is initialized.*/
 
             /*signal the parent process.*/
-            if (parentPid != getpid()){
+            if (parentPid != getpid()) {
                 daemonize_signal_parent(parentPid);
             }
 #endif
         }
 
+#elif defined(_WIN32)
+        if (optget(opts, "service-mode")->enabled) {
+            cl_engine_set_clcb_sigload(engine, NULL, NULL);
+            svc_ready();
+        }
 #endif
 
         if (nlsockets == 0) {
